%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This is an example to demonstrate the usage of convmtxSV.m
% 
% Reference:
% Stanley H. Chan, "Constructing a sparse convolution matrix for shift
% varying image restoration problems", in Proceedings of IEEE International
% Conference on Image Processing (ICIP' 10), 2010.
%
% Stanley Chan
% University of California, San Diego
% 
% Last modified: 
% 2011-Mar-01
% 2010-Oct-11
% 2013-Jan-3
% 2016-Aug-23
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clc
clear all
close all

% Load Image
x = im2double(imread('cameraman.tif'));
x = imresize(x,[150,200]);
[M, N] = size(x);

% Define spatially variant blurs
P = 13; % P, Q must be odd
Q = 5;

h{1} = fspecial('gaussian', [P, Q], 1);
idxY{1} = 1:100;
idxX{1} = 1:100;

h{2} = fspecial('gaussian', [P, Q], 3);
idxY{2} = 101:150;
idxX{2} = 1:100;

h{3} = fspecial('gaussian', [P, Q], 5);
idxY{3} = 1:100;
idxX{3} = 101:200;

h{4} = fspecial('gaussian', [P, Q], 7);
idxY{4} = 101:150;
idxX{4} = 101:200;


%% Experiment 1: Convolution by y = Tx;
tic
T = convmtxSV(h,idxY,idxX,M,N,P,Q);
toc
y = T*x(:);
Y = reshape(y,M,N);


%% Experiment 2: Convolution by z = h1*x + h2*x + h3*x + h4*x;
Z1 = imfilter(x,h{1},'circular');
Z2 = imfilter(x,h{2},'circular');
Z3 = imfilter(x,h{3},'circular');
Z4 = imfilter(x,h{4},'circular');
Z  = zeros(M,N);
Z(idxY{1},idxX{1})   = Z1(idxY{1},idxX{1});
Z(idxY{2},idxX{2})   = Z2(idxY{2},idxX{2});
Z(idxY{3},idxX{3})   = Z3(idxY{3},idxX{3});
Z(idxY{4},idxX{4})   = Z4(idxY{4},idxX{4});


%% Display Results
figure;
subplot(131);imshow(Y,[]);title('Blurred by y = Tx');
subplot(132);imshow(Z,[]);title('Blurred by z = h1*x + h2*x + h3*x + h4*x');
subplot(133);imshow(Y-Z,[]);title(['Difference, ||y-z||_\infty = ', num2str(max(abs(Y(:)-Z(:))))]);